*:*********************************************************************
*:
*:        Program: METER.PRG
*:
*:         System: Hobbitronic's Limited - Function Library
*:         Author: Donald R. Stark
*:      Copyright (c) 1990, Hobbitronic's Ltd.
*:  Last modified: 12/09/90     16:30
*:
*:  Procs & Fncts: NTX_METER()
*:               : NTX_BAR()
*:
*:     Documented: 12/09/90 at 17:00                SNAP!  version 4.00
*:*********************************************************************

*!*********************************************************************
*!
*!       Function: NTX_METER()
*!
**    Description: Create an Index while displaying a indexing status bar
**
*!          Calls: NTX_BAR()      (function  in METER.PRG)
*!
*!           Uses: &DBF_NAME
*!
*!        Indexes: &NTX_NAME
*!
**         Syntax: NTX_METER(<expC1>, <expC2>, <expC3>, [<expN1>])
**
**     Parameters: <expC1>   Name of the database file to be indexed.
**                           If the extension is omitted, .DBF is assumed.
**
**                 <expC2>   Name of the target index file to create.
**                           If the extension is omitted, .NTX is assumed.
**
**                 <expC3>   Expression to use to perform the index
**                           (index key)
**
**                 [<expN1>] Top row of index box, if omitted ROW 8.
**                           Adjusted to ROW 1 if ROW 0 is passed
**                           Adjusted to ROW 20 if a ROW > 20 is passed
**
**        Returns: Logical representing success of the index procedure
**
**       Warnings: Works Only for character based indices
**
**  LIBS Required: CLIPPER.LIB, EXTEND.LIB, SUPER.LIB
**
*!*********************************************************************
FUNCTION ntx_meter
PARAMETERS dbf_name, ntx_name, ntx_expr, ntx_row
PRIVATE ntx_color, c_title, ntx_offset, ntx_msg, ntx_handle, ntx_fill
IF Pcount() < 3
  Msg("Must Pass The Following Parameters To Ntx_meter():",;
    "",;
    "     <expC1>   - Database Name",;
    "     <expC2>   - Index Name",;
    "     <expC3>   - Index Expression",;
    "     [<expN1>] - Index Box Top Row (Optional)")
  RETURN (.F.)
ENDIF

ntx_color  = SETCOLOR()                    && Save The Current Color
ntx_area   = SELECT()                      && Save The Curent Area
ntx_row    = IIF(Pcount() < 4, 8, IIF(m->ntx_row < 1, 1, IIF(m->ntx_row > 20, 20, m->ntx_row)))
ntx_offset = LEN(ntx_expr)+22
ntx_name   = IIF(AT('.', ntx_name) = 0, ntx_name+'.NTX', ntx_name)
ntx_expr   = ntx_expr + '+ntx_bar()'

SELECT 0
USE &dbf_name

ntx_msg1 = '[ Creating Index: '+UPPER(m->ntx_name)+' ]'
ntx_msg2 = '[ Total # of Records: '+LTRIM(TRANSFORM(LASTREC(),'999,999,999'))+' ]'
ntx_box  = Makebox(m->ntx_row,11,m->ntx_row+3,68, m->c_popcol, m->c_shadpos)
@ m->ntx_row+2, 14 SAY '0  10  20  30  40  50  60  70  80  90  100%'
Prnt(m->ntx_row,  (40 - LEN(m->ntx_msg1)/2), m->ntx_msg1, Enhanced())
Prnt(m->ntx_row+3,(40 - LEN(m->ntx_msg2)/2), m->ntx_msg2, Enhanced())

INDEX ON &ntx_expr TO &ntx_name            && Make The Index
USE                                        && Close The Database

ntx_handle = FOPEN("&ntx_name",1)          && Get The File Handle Of Index File
ntx_fill   = REPLICATE(CHR(0),10)
Fseek(ntx_handle, ntx_offset, 0)           && Move Pointer X Bytes From Bof
ret_val = IIF(Fwrite(ntx_handle, ntx_fill, 10)#10, .F., .T.)
Fclose(ntx_handle)                         && Close File

Unbox(m->ntx_box)                          && Cleanup and go home
SETCOLOR(m->ntx_color)                     && Reset the screen color
SELECT (m->ntx_area)                       && Reset the area

RETURN(ret_val)

*!*********************************************************************
*!
*!       Function: NTX_BAR()
*!
**    Description: Display the Actual Index Bar
**
*!      Called by: NTX_METER()    (function  in METER.PRG)
*!
*!*********************************************************************
FUNCTION ntx_bar
@ m->ntx_row+1, 15 SAY IIF(RECNO() <= LASTREC(), REPLICATE(CHR(8), ;
  (RECNO()/LASTREC())*100/2), IIF(LASTREC() = 0, REPLICATE(CHR(8), 50),''))
RETURN('')

*: EOF: METER.PRG
